from vcCommand import *

app = getApplication()
teach = getApplication().TeachContext

def OnStart():
  routine = teach.ActiveScope
  
  processStatement = routine.addStatement(VC_STATEMENT_PROCESS, teach.ActiveStatementIndex + 1)
    
  name = processStatement.getProperty('Name')
  base = processStatement.getProperty('Base')
  tool = processStatement.getProperty('Tool')
  
  processStatement.createProperty(VC_VECTOR, 'ParentVector')
  processStatement.createProperty(VC_STRING, 'ParentName')
  processStatement.createProperty(VC_BUTTON, 'SetParentVector')  
  
  processStatement.createProperty(VC_VECTOR, 'ChildVector')
  processStatement.createProperty(VC_STRING, 'ChildName')
  processStatement.createProperty(VC_BUTTON, 'SetChildVector')    
  
  robot = teach.ActiveRobot 
  attachStatementScriptName = 'AttachStatementScript_' + processStatement.Name
  attachStatementScript = robot.findBehaviour(attachStatementScriptName)
  if not attachStatementScript:
    attachStatementScript = robot.createBehaviour(VC_PYTHONPROCESSHANDLER, attachStatementScriptName)
  attachStatementScript.Script = '''
from vcPythonProcessHandler import *
import vcVector

app = getApplication()
snap = app.findCommand('interactiveSnap') 

def ParentPicked(arg):
  global parentVector, parentName
  target = snap.TargetNode.Component  
  parentVector.Value = target.WorldPositionMatrix.P
  parentName.Value = target.Name

def ChildPicked(arg):
  global childVector, childName
  target = snap.TargetNode.Component  
  childVector.Value = target.WorldPositionMatrix.P
  childName.Value = target.Name
  
def OnPickComponent(arg):
  snap.clear() 
  if 'Parent' in arg.Name:
    snap.OnTargetSet = ParentPicked   
  elif 'Child' in arg.Name:
    snap.OnTargetSet = ChildPicked   
  snap.execute()

def hideOthers(statement):
  name = statement.getProperty('Name')
  name.IsVisible = False
  base = statement.getProperty('Base')
  base.IsVisible = False
  tool = statement.getProperty('Tool')
  tool.IsVisible = False

def setEvents(statement):
  global parentVector, parentName, childVector, childName
  
  parentVector = statement.getProperty('ParentVector')
  parentName = statement.getProperty('ParentName')
  childVector = statement.getProperty('ChildVector')
  childName = statement.getProperty('ChildName')
  
  global setParentVector, setChildVector
  setParentVector = statement.getProperty('SetParentVector')
  if setParentVector:
    if not setParentVector.OnChanged:
      setParentVector.OnChanged = OnPickComponent  
  setChildVector = statement.getProperty('SetChildVector')
  if setChildVector:
    if not setChildVector.OnChanged:
      setChildVector.OnChanged = OnPickComponent 

def OnStatementAdd(statement):
  # Hide others
  hideOthers(statement)
  
  # Set events
  setEvents(statement)

def OnStatementModified(statement):
  # Hide others
  hideOthers(statement)
  
  # Set events
  setEvents(statement)
  
def OnStatementExecute(executor, statement):  
  # Hide others
  hideOthers(statement)
  
  # Set events
  setEvents(statement)
  
  parent = None
  child = None
  for part in app.Components:    
    if parent == None and part.Name == parentName.Value and (part.WorldPositionMatrix.P - parentVector.Value).length() < 1e-3:
      parent = part         
    if child == None and part.Name == childName.Value and (part.WorldPositionMatrix.P - childVector.Value).length() < 1e-3:
      child = part    
    if parent and child:      
      break
  
  if parent and child:    
    matrix = parent.InverseWorldPositionMatrix * child.WorldPositionMatrix
    parent.attach(child)
    child.PositionMatrix = matrix
'''
  processStatement.Process = attachStatementScript
  app.SelectionManager.setSelection(processStatement)
